// DTMF_Bios_FFT.c  
// This program recognizes DTMF signals and lights corresponding LEDs.
// To determine the dominating frequencies, the FFT algorithm is used.
// As input, it may use, MATLAB generated signals, Dialpad or a microphone.


#include "dsk6416_aic23.h"
#include <math.h>
#include <rtdx.h>

#define N 256  		           		// specifies number of complex FFT points
#define II 3.14159265359			// define the time honored pi
#define THRESHOLD_VALUE	1.0e3

// These are defined as N/fs*frequency or equivalently 256/8000*dtmf_frequency
#define DTMF697		22
#define DTMF770		25
#define DTMF852		27
#define DTMF941		30
#define DTMF1209	39
#define DTMF1336	43
#define DTMF1477	47

#pragma DATA_ALIGN(x, 8);      		// double-word aligned
#pragma DATA_ALIGN(w, 8);

Uint32 fs=DSK6416_AIC23_FREQ_8KHZ;

float x[2*N];               		// FFT i/p: real/img consecutively aligned
float w[3*N/2];                    	// Complex twiddle factors
short input_buffer[N] = {0};		// used to store input samples...same as x
float output_buffer[7] = {0};		// used to store the magnitude of the FFT

int count; 							// used in digit reversal
short buffer_count;
short nFlag;						// used as an indicator to begin the FFT
short i, j;
short nRow, nColumn;
short value;
double delta;
float tempvalue;

unsigned short JIndex[4*N];
unsigned short IIndex[4*N];

RTDX_CreateOutputChannel(output_channel); //create output channel for C6x-->PC data transfer


// interrupt service routine
interrupt void c_int11()
{
	input_buffer[buffer_count] = input_sample();
	output_sample((int)input_buffer[buffer_count++]);

	// if DSK has accummulated more than N points, it buffers the accummulated
	// samples and begins the computation of the FFT upon return from the interrupt
	if(buffer_count >= N)
	{
		buffer_count = 0;					// reset buffer_count
		nFlag = 0;							// flag to signal completion

		for(i = 0; i < N; i++)
		{
			x[2*i] = (float)input_buffer[i]; // real part of input
			x[2*i+1] = 0;					// imaginary part of input
		}
	}
}

void main(void)
{
	delta = 2*II/N;
	nFlag = 1;
	buffer_count = 0;

    // generate twiddle constants
    for(j = 0; j < 3*N/4; j++)
    {
        w[2*j+1] = sin(j*delta);
        w[2*j] = cos(j*delta);
    }

    // generate index for FFT digit reversal
	R4DigitRevIndexTableGen(N, &count, IIndex, JIndex);
	
	DSK6416_LED_init();
	DSK6416_LED_off(0);
	DSK6416_LED_off(1);
  	DSK6416_LED_off(2);
  	DSK6416_LED_off(3);

	// initialize the DSK board
	comm_intr();

	while(!RTDX_isOutputEnabled(&output_channel)) 		//wait for PC to enable RTDX
 	   puts("\nWaiting for PC to enable RTDX..... ");

	// infinite loop
	while(1)
	{
    	while(nFlag); // wait for interrupt to finish buffering accummulated samples
    	nFlag = 1;

    	/* ----------------------------------------------------------------------- */
		/*  Perform complex FFT. Output in array x                        	       */
		/* ----------------------------------------------------------------------- */
		//  call optimized assembly function
		cfftr4_dif(x, w, N);
		digit_reverse((double *)x, IIndex, JIndex, count);

    	// Row Frequencies
    	output_buffer[0] = (float) sqrt(x[2*DTMF697]*x[2*DTMF697] + x[2*DTMF697+1]*x[2*DTMF697+1]);
    	output_buffer[1] = (float) sqrt(x[2*DTMF770]*x[2*DTMF770] + x[2*DTMF770+1]*x[2*DTMF770+1]);
    	output_buffer[2] = (float) sqrt(x[2*DTMF852]*x[2*DTMF852] + x[2*DTMF852+1]*x[2*DTMF852+1]);
    	output_buffer[3] = (float) sqrt(x[2*DTMF941]*x[2*DTMF941] + x[2*DTMF941+1]*x[2*DTMF941+1]);
    	
    	// Column Frequencies
    	output_buffer[4] = (float) sqrt(x[2*DTMF1209]*x[2*DTMF1209] + x[2*DTMF1209+1]*x[2*DTMF1209+1]);
    	output_buffer[5] = (float) sqrt(x[2*DTMF1336]*x[2*DTMF1336] + x[2*DTMF1336+1]*x[2*DTMF1336+1]);
    	output_buffer[6] = (float) sqrt(x[2*DTMF1477]*x[2*DTMF1477] + x[2*DTMF1477+1]*x[2*DTMF1477+1]);

		// choose the largest row frequency
		tempvalue = 0;
		nRow = 0;
		for(j = 0; j < 4; j++)
		{
			if(tempvalue < output_buffer[j])
			{
				if(output_buffer[j] > THRESHOLD_VALUE)
				{
					nRow = j + 1;
					tempvalue = output_buffer[j];
				}
			} 
		}
		
		// choose the largest column frequency
		tempvalue = 0;
		nColumn = 0;
		for(j = 4; j < 7; j++)
		{
			if(tempvalue < output_buffer[j])
			{
				if(output_buffer[j] > THRESHOLD_VALUE)
				{
					nColumn = j - 3;
					tempvalue = output_buffer[j];
				}
			} 
		}
		
		if((nRow != 0) && (nColumn != 0))
		{
		  	// depending on choice 1 and choice 2 light corresponding LEDs
			if((nRow==1)&&(nColumn==1))
			{
				DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_on(3);
				value = 1;
			}
			else if((nRow==1)&&(nColumn==2))	// button 2  -  0010
			{
				DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_off(3);
				value = 2;
			}
			else if((nRow==1)&&(nColumn==3))	// button 3  -  0011
			{
				DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_on(3);
				value = 3;
			}
			else if((nRow==2)&&(nColumn==1))	// button 4  -  0100
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
				value = 4;
			}
			else if((nRow==2)&&(nColumn==2))	// button 5  -  0101
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_off(2); DSK6416_LED_on(3);
				value = 5;
			}
			else if((nRow==2)&&(nColumn==3))	// button 6  -  0110
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_on(2); DSK6416_LED_off(3);
				value = 6;
			}
			else if((nRow==3)&&(nColumn==1))	// button 7  -  0111
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_on(2); DSK6416_LED_on(3);
				value = 7;
			}
			else if((nRow==3)&&(nColumn==2))	// button 8  -  1000
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
				value = 8;
			}
			else if((nRow==3)&&(nColumn==3))	// button 9  -  1001
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_on(3);
				value = 9;
			}
			else if((nRow==4)&&(nColumn==1))	// button *  -  1010
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_off(3);
				value = 10;
			}
			else if((nRow==4)&&(nColumn==2))	// button 0  -  1011
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_on(3);
				value = 11;
			}
			else if((nRow==4)&&(nColumn==3))	// button #  -  1100
			{
				DSK6416_LED_on(0); DSK6416_LED_on(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
				value = 12;
			}

		 }
		 else
		 {
		 	// if weights are below the threshold, turn LEDs off
		 	DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
		 	value = 0;
		 }

		 RTDX_write(&output_channel, &value, sizeof(value)); //write data to PC
			
	};
}

