// DTMF_GOERTZEL.c
// This program recognizes DTMF signals and lights corresponding LEDs.
// To determine the dominating frequencies, the Goertzel algorithm is used.
// As input, it may use MATLAB generated signals, Dialpad or a microphone
// connected to a touch tone phone.

#include "dsk6416_aic23.h"		//codec-DSK support file
#include <math.h>
#include <rtdx.h>

#define II	3.141592653
#define N	205
#define THRESHOLD_VALUE	1000000.0	// define accordingly to suppress noise

Uint32 fs=DSK6416_AIC23_FREQ_8KHZ;	//set sampling rate

float buffer[N] = {0.0};
float weights[7];

// Predefined values for the seven DTMF frequencies. k = 0.5 + N*frequency/sampling rate
float k[7] = {17.861, 19.731, 21.833, 24.113, 30.981, 34.235, 37.848};

float W0;
float W[7][2] = {0.0};
float Magnitude[7] = {0.0};
float temp;
float tempMagnitude;
short input_buffer;
short counter = 0;
short process = 0;
short i, j;
short nRow, nColumn;
short value = 0;

RTDX_CreateOutputChannel(output_channel); //create output channel for C6x-->PC data transfer

void zero_out()
{
	for(i = 0; i < 7; i++)
		for(j = 0; j < 2; j++)
			W[i][j] = 0.0;

	return;
}

interrupt void c_int11()
{
	input_buffer = input_sample();
	output_sample(input_buffer);

	buffer[counter] = (float)input_buffer;

	counter++;
	if(counter >= N)
	{
		counter = 0;
		process = 1;
	}
	else
		process = 0;

	return;
}

void main()
{
	// Calculate weights for the seven different DTMF frequencies
	for(i = 0; i < 7; i++)
	{
		temp = 2.0*II*k[i]/N;
		weights[i] = 2.0*cos(temp);
	}

	DSK6416_LED_init();
	DSK6416_LED_off(0);
	DSK6416_LED_off(1);
  	DSK6416_LED_off(2);
  	DSK6416_LED_off(3);

	comm_intr();				     				//init DSK, codec, McBSP

	while(!RTDX_isOutputEnabled(&output_channel)) 		//wait for PC to enable RTDX
 	   puts("\nWaiting for PC to enable RTDX..... ");

	while(1)
	{
		while(!process);

		zero_out();

		for(i = 0; i < N; i++)
		{
			for(j = 0; j < 7; j++)
			{
				W0 = weights[j]*W[j][0] - W[j][1] + buffer[i];
				W[j][1] = W[j][0];
				W[j][0] = W0;
			}
		}

		for(i = 0; i < 7; i++)
			Magnitude[i] = W[i][0]*W[i][0] - W[1][0]*W[i][1] + W[i][1]*W[i][1];

		// choose the largest row frequency
		tempMagnitude = 0.0;
		nRow = 0;
		for(j = 0; j < 4; j++)
		{
			if(tempMagnitude < Magnitude[j])
			{
				if(Magnitude[j] > THRESHOLD_VALUE)
				{
					nRow = j + 1;
					tempMagnitude = Magnitude[j];
				}
			}
		}

		// choose the largest column frequency
		tempMagnitude = 0.0;
		nColumn = 0;
		for(j = 4; j < 7; j++)
		{
			if(tempMagnitude < Magnitude[j])
			{
				if(Magnitude[j] > THRESHOLD_VALUE)
				{
					nColumn = j - 3;
					tempMagnitude = Magnitude[j];
				}
			}
		}

		if((nRow != 0) && (nColumn != 0))
		{
		  	// depending on choice 1 and choice 2 light corresponding LEDs
			if((nRow==1)&&(nColumn==1))
			{
				DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_on(3);
				value = 1;
			}
			else if((nRow==1)&&(nColumn==2))	// button 2  -  0010
			{
				DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_off(3);
				value = 2;
			}
			else if((nRow==1)&&(nColumn==3))	// button 3  -  0011
			{
				DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_on(3);
				value = 3;
			}
			else if((nRow==2)&&(nColumn==1))	// button 4  -  0100
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
				value = 4;
			}
			else if((nRow==2)&&(nColumn==2))	// button 5  -  0101
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_off(2); DSK6416_LED_on(3);
				value = 5;
			}
			else if((nRow==2)&&(nColumn==3))	// button 6  -  0110
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_on(2); DSK6416_LED_off(3);
				value = 6;
			}
			else if((nRow==3)&&(nColumn==1))	// button 7  -  0111
			{
				DSK6416_LED_off(0); DSK6416_LED_on(1); DSK6416_LED_on(2); DSK6416_LED_on(3);
				value = 7;
			}
			else if((nRow==3)&&(nColumn==2))	// button 8  -  1000
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
				value = 8;
			}
			else if((nRow==3)&&(nColumn==3))	// button 9  -  1001
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_on(3);
				value = 9;
			}
			else if((nRow==4)&&(nColumn==1))	// button *  -  1010
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_off(3);
				value = 10;
			}
			else if((nRow==4)&&(nColumn==2))	// button 0  -  1011
			{
				DSK6416_LED_on(0); DSK6416_LED_off(1); DSK6416_LED_on(2); DSK6416_LED_on(3);
				value = 11;
			}
			else	// button #  -  1100
			{
				DSK6416_LED_on(0); DSK6416_LED_on(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
				value = 12;
			}

		 }
		 else
		 {
		 	// if weights are below the threshold, turn LEDs off
		 	DSK6416_LED_off(0); DSK6416_LED_off(1); DSK6416_LED_off(2); DSK6416_LED_off(3);
		 	value = 0;
		 }

		 RTDX_write(&output_channel, &value, sizeof(value)); //write data to PC

	};	//infinite loop
}

